.sidebar {
    border-width: 1px 0 0 0;
    border-style: solid;
    border-color: shade (@theme_bg_color, 0.66);
    border-radius: 0;
}

.pane-separator {
    border-width: 1px 1px 0 0;
    border-style: solid;
    border-color: shade (@theme_bg_color, 0.66);
}
